// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf.bind;

import 'package:shelf/shelf.dart';
import 'package:shelf_route/shelf_route.dart';
import 'dart:mirrors';
import 'src/preconditions.dart';
import 'dart:async';
import 'dart:convert';

Handler bind(Type type, Map<String, Symbol> bindings, Function handler,
             {Symbol constructor: #build}) {
  return new _RequestParamBinding(type, constructor, bindings, handler).handler;
}

Handler bindJsonBody(Type type, Function handler) {
  return new _JsonBodyBinding(type, handler).handler;
}

class _JsonBodyBinding {
  final Type type;
  final Function boundHandler;

  Handler get handler => _handleRequest;

  _JsonBodyBinding(this.type, this.boundHandler) {
    ensure(type, isNotNull);
    ensure(boundHandler, isNotNull);
  }

  dynamic _handleRequest(Request request) {
    final jsonFuture = _extractJsonBody(request);
    return jsonFuture.then((json) {
      final classMirror = reflectClass(type);
      final instanceMirror =
          classMirror.newInstance(#fromJson, [json]);

      return boundHandler(instanceMirror.reflectee, request);
    });
  }

  Future<Map> _extractJsonBody(Request request) =>
      request.readAsString().then((s) => JSON.decode(s));

}


class _RequestParamBinding {
  final Type type;
  final Symbol constructorName;
  final Map<String, Symbol> memberBindings;
  final Function boundHandler;

  Handler get handler => _handleRequest;

  _RequestParamBinding(this.type, this.constructorName, this.memberBindings,
      this.boundHandler) {
    ensure(type, isNotNull);
    ensure(constructorName, isNotNull);
    ensure(memberBindings, isNot(isEmpty));
    ensure(boundHandler, isNotNull);
  }

  dynamic _handleRequest(Request request) {
    final namedArguments = _bindVariables(request);
    final classMirror = reflectClass(type);
    final instanceMirror =
        classMirror.newInstance(constructorName, [], namedArguments);

    return boundHandler(instanceMirror.reflectee, request);
  }

  Map<Symbol, dynamic> _bindVariables(Request request) {
    final pv = getPathVariables(request);
    final namedArguments = <Symbol, dynamic> {};
    memberBindings.forEach((k, v) {
      final value = pv[k];
      if (value != null) {
        namedArguments[v] = value;
      }
    });

    return namedArguments;
  }
}


