// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf.bind.test;

import 'package:shelf_bind/shelf_bind.dart' as sb;
import 'package:shelf/shelf.dart' as s;
import 'package:shelf_route/shelf_route.dart' as sr;


import 'package:unittest/unittest.dart';
import 'dart:convert';
import 'dart:async';

//TODO:real tests
main() {
  test('', (){
    final Uri uri = Uri.parse('http://localhost/person/fred');
    final bindHandler = sb.bind(Person, {'name': #name}, _handlePerson);
    final routeHandler = (sr.router()..get('/person/{name}', bindHandler)).handler;

    final request = new s.Request('GET', uri);
    routeHandler(request);
  });

  test('', (){
    final Uri uri = Uri.parse('http://localhost/person?name=fred');
    final handler = sb.bind(Person, {'name': #name}, _handlePerson);
    final router = (sr.router()..get('/person{?name}', handler)).handler;

    final request = new s.Request('GET', uri);
    router(request);
  });

  test('', (){
    final Uri uri = Uri.parse('http://localhost/person');
    final handler = sb.bindJsonBody(Person, _handlePerson);
    final router = (sr.router()..post('/person', handler)).handler;

    final person = new Person('fred');
    final json = JSON.encode(person.toJson());
    final body = new Future.sync(() => json.codeUnits).asStream();
    final request = new s.Request('POST', uri, body: body);
    router(request);
  });
}

s.Response _handlePerson(Person person, s.Request request) {
  print(person);
  return new s.Response.ok('yippee - ${person.toJson()}');
}


class Person {
  final String name;

  Person(this.name);

  Person.build({String name}) : this(name);

  Person.fromJson(Map json) : this(json['name']);

  Map toJson() => { 'name': name };

  String toString() => 'Person[name: $name]';
}